#pragma ident "$Id: main.c,v 1.11 2004/09/29 18:22:51 dechavez Exp $"
/*======================================================================
 *
 *  General purpose ISI client
 *
 *====================================================================*/
#include "isi.h"
#include "util.h"

extern char *VersionIdentString;

#ifndef DEFAULT_SERVER
#define DEFAULT_SERVER "localhost"
#endif /* DEFAULT_SERVER */

#define UNKNOWN 0
#define DATA 1
#define CNF  2
#define SOH  3
#define WD   4

BOOL verbose   = FALSE;
char *server   = DEFAULT_SERVER;
char *datdir   = NULL;

static void WriteFrame(ISI_GENERIC_TS *ts)
{
FILE *fp;
char parent[MAXPATHLEN+1], path[MAXPATHLEN+1];

    mkdir(datdir, 0775);
    sprintf(parent, "%s/%s", datdir, ts->hdr.name.sta);
    mkdir(parent, 0775);

    if (strlen(ts->hdr.name.loc) != 0) {
        sprintf(path, "%s/%s%s", parent, ts->hdr.name.chn, ts->hdr.name.loc);
    } else {
        sprintf(path, "%s/%s", parent, ts->hdr.name.chn);
    }

    if ((fp = fopen(path, "ab")) == NULL) {
        fprintf(stderr, "fopen: ");
        perror(path);
        exit(1);
    }

/* non-native packets include the generic header */

    if (!isiIsNative(&ts->hdr.desc)) {
        int len;
        char buf[sizeof(ISI_GENERIC_TSHDR)*2];
        len = isiPackGenericTSHDR((UINT8 *) buf, &ts->hdr);
        if ((fwrite(buf, sizeof(UINT8), len, fp)) != len) {
            fprintf(stderr, "fwrite: ");
            perror(path);
            exit(1);
        }
    }

/* Save the payload */

    if ((fwrite(ts->data, sizeof(UINT8), ts->hdr.nbytes, fp)) != ts->hdr.nbytes) {
        fprintf(stderr, "fwrite: ");
        perror(path);
        exit(1);
    }
    fclose(fp);
}

static ISI_DATA_REQUEST *BuildDataRequest(int format, int compress, REAL64 beg, REAL64 end, char *StreamSpec)
{
ISI_DATA_REQUEST *dreq;

    if ((dreq = isiAllocSimpleDataRequest(beg, end, StreamSpec)) == NULL) {
        fprintf(stderr, "isiAllocSimpleDataRequest: %s\n", strerror(errno));
        exit(1);
    }
    isiSetDatreqFormat(dreq, format);
    isiSetDatreqCompress(dreq, compress);

    return dreq;
}

static void data(char *server, ISI_PARAM *par, int format, int compress, REAL64 beg, REAL64 end, char *StreamSpec)
{
ISI *isi;
int status;
ISI_GENERIC_TS *ts;
ISI_DATA_REQUEST *dreq;
#define LOCALBUFLEN 1024
UINT8 buf[LOCALBUFLEN];

    dreq = BuildDataRequest(format, compress, beg, end, StreamSpec);

    if (verbose) {
        fprintf(stderr, "Client side data request\n");
        isiPrintDatreq(stderr, dreq);
    }

    if ((isi = isiInitiateDataRequest(server, par, dreq)) == NULL) {
        perror("isiInitiateDataRequest");
        exit(1);
    }

    isiFreeDataRequest(dreq);

    if (verbose) {
        fprintf(stderr, "Server expanded data request\n");
        isiPrintDatreq(stderr, &isi->datreq);
    }

    while ((ts = isiReadGenericTS(isi, &status)) != NULL) {
        fprintf(stderr, "%s\n", isiGenericTsHdrString(&ts->hdr, (char *) buf));
        if (datdir != NULL) WriteFrame(ts);
    }

    if (status != ISI_DONE) perror("isiReadGenericTS");
}

static void soh(char *server, ISI_PARAM *par)
{
int i;
ISI_SOH_REPORT *report;

    if ((report = isiSoh(server, par)) == NULL) {
        perror("isiSoh");
        exit(1);
    }

    if (verbose) printf(
        "  Sta Chn Loc  Nseg     Nrec  Oldest Sample Time   "
        "Youngest Sample Time     Last Write\n"
    );

    for (i = 0; i < report->nentry; i++) isiPrintStreamSoh(stdout, &report->entry[i]);
    isiFreeSoh(report);
}

static void cnf(char *server, ISI_PARAM *par)
{
int i;
ISI_CNF_REPORT *report;

    if ((report = isiCnf(server, par)) == NULL) {
        perror("isiCnf");
        exit(1);
    }

    if (verbose) printf(
        "  Sta Chn Loc    Sint    Lat       Long        Elev"
        "     Depth     Calib      Calper    Hang    Vang   Inst\n"
    );

    for (i = 0; i < report->nentry; i++) isiPrintStreamCnf(stdout, &report->entry[i]);
    isiFreeCnf(report);
}

static void wd(char *server, ISI_PARAM *par, int maxdur)
{
int i;
ISI_WFDISC_REPORT *report;

    if ((report = isiWfdisc(server, par, maxdur)) == NULL) {
        perror("isiWfdisc");
        exit(1);
    }

    for (i = 0; i < report->nentry; i++) printf("%s\n", report->entry[i]);
    isiFreeWfdisc(report);
}

static void help(char *myname)
{
static char *VerboseHelp = 
"The arguments in [ square brackets ] are optional:\n"
"\n"
"           -v - turns on verbose isi output\n"
"server=string - sets the name of the server to the specified string.\n"
"     port=int - sets the port number\n"
"   log=string - turns on library logging using. 'string' is either the\n"
"                name of the log file, '-' for stdout, or 'syslogd:facility'\n"
"    debug=int - sets the debug level\n"
"\n"
"You must specify ONE of the arguments in { curly brackets }.\n"
"\n"
"    cnf - configuration report\n"
"    soh - state of health report\n"
"     wd - disk loop wfdisc dump\n"
"   data - waveform request\n"
"\n"
"Data feeds are done using the 'data' argument.  By itself, the data request\n"
"defaults to all streams, starting with the most current data and continuing\n"
"indefinitely.  The optional '=spec' suffix allows you to select specific \n"
"streams.  The format of the 'spec' string is dot delimited 'sta.chn.loc'.\n"
"If you use wild cards, be sure to quote the string so the shell won't get\n"
"confused.  You may string together multiple spec strings, delimited by a\n"
"plus (sta1.chn.loc+sta2.chn.loc, etc).  The 'datdir' argument is used to\n"
"to save the data frames in the indicated directory.  Without 'datdir' then\n"
"all you get are packet header summaries printed to stderr.";

    fprintf(stderr,"usage: %s ", myname);
    fprintf(stderr, "[ -v server=string port=int log=string debug=int ] ");
    fprintf(stderr, "{ data[=spec] [datdir=path] | cnf | soh | wd }\n", myname);
    fprintf(stderr, "\n");
    fprintf(stderr, "%s\n", VerboseHelp);
    fprintf(stderr, "\n");
    fprintf(stderr, "default server is `%s'\n", DEFAULT_SERVER);
    exit(1);

}

int main (int argc, char **argv)
{
int i, request, port, debug;
ISI_PARAM par;
char *req = NULL;
char *log;
char *StreamSpec = NULL;
REAL64 beg = ISI_NEWEST;
REAL64 end = ISI_KEEPUP;
char *tmpstr;
int maxdur    = 0;
int compress  = ISI_COMP_NONE;
int format    = ISI_FORMAT_GENERIC;

    utilNetworkInit();
    isiInitDefaultPar(&par);

    for (i = 1; i < argc; i++) {
        if (strncmp(argv[i], "server=", strlen("server=")) == 0) {
            server = argv[i] + strlen("server=");
        } else if (strncmp(argv[i], "port=", strlen("port=")) == 0) {
            port = atoi(argv[i] + strlen("port="));
            isiSetServerPort(&par, port);
        } else if (strncmp(argv[i], "beg=", strlen("beg=")) == 0) {
            beg = utilAttodt(argv[i] + strlen("beg="));
        } else if (strncmp(argv[i], "end=", strlen("end=")) == 0) {
            end = utilAttodt(argv[i] + strlen("end="));
        } else if (strncmp(argv[i], "format=", strlen("format=")) == 0) {
            tmpstr = argv[i] + strlen("format=");
            if (strcasecmp(tmpstr, "native") == 0) {
                format = ISI_FORMAT_NATIVE;
            } else if (strcasecmp(tmpstr, "generic") == 0) {
                format = ISI_FORMAT_GENERIC;
            } else {
                fprintf(stderr, "%s: unsupported format type '%s'\n", argv[0], tmpstr);
                help(argv[0]);
            }
        } else if (strncmp(argv[i], "comp=", strlen("comp=")) == 0) {
            tmpstr = argv[i] + strlen("comp=");
            if (strcasecmp(tmpstr, "none") == 0) {
                compress = ISI_COMP_NONE;
            } else if (strcasecmp(tmpstr, "ida") == 0) {
                compress = ISI_COMP_IDA;
            } else {
                fprintf(stderr, "%s: unsupported comp type '%s'\n", argv[0], tmpstr);
                help(argv[0]);
            }
        } else if (strncmp(argv[i], "datdir=", strlen("datdir=")) == 0) {
            datdir = argv[i] + strlen("datdir=");
        } else if (strncmp(argv[i], "debug=", strlen("debug=")) == 0) {
            debug = atoi(argv[i] + strlen("debug="));
            isiSetDebugFlag(&par, debug);
        } else if (strncmp(argv[i], "maxdur=", strlen("maxdur=")) == 0) {
            maxdur = atoi(argv[i] + strlen("maxdur="));
        } else if (strncmp(argv[i], "log=", strlen("log=")) == 0) {
            log = argv[i] + strlen("log=");
            isiSetLogging(&par, log, NULL, argv[0]);
        } else if (strcmp(argv[i], "-v") == 0) {
            verbose = TRUE;
        } else if (req == NULL) {
            req = argv[i];
        } else {
           fprintf(stderr, "%s: unrecognized argument: '%s'\n", argv[0], argv[i]);
            help(argv[0]);
        }
    }

    if (req == NULL) help(argv[0]);

    if (verbose) fprintf(stderr, "%s %s\n", argv[0], VersionIdentString);

    request = UNKNOWN;
    if (strcasecmp(req, "data") == 0) {
        request = DATA;
        StreamSpec = NULL;
    } else if (strncasecmp(req, "data=", strlen("data=")) == 0) {
        request = DATA;
        StreamSpec = req + strlen("data=");
        if (strlen(StreamSpec) == 0) StreamSpec = NULL;
    } else if (strcmp(req, "cnf") == 0) {
        request = CNF;
    } else if (strcmp(req, "soh") == 0) {
        request = SOH;
    } else if (strcmp(req, "wd") == 0) {
        request = WD;
    } else if (strcmp(req, "wfdisc") == 0) {
        request = WD;
    } else {
        help(argv[0]);
    }

    switch (request) {
      case DATA: 
        if (verbose) fprintf(stderr, "%s data request\n", server);
        data(server, &par, format, compress, beg, end, StreamSpec);
        break;
      case CNF:
        if (verbose) fprintf(stderr, "%s configuration request\n", server);
        cnf(server, &par);
        break;
      case SOH:
        if (verbose) fprintf(stderr, "%s state of health request\n", server);
        soh(server, &par);
        break;
      case  WD:
        if (verbose) fprintf(stderr, "%s wfdisc request\n", server);
        wd(server, &par, maxdur);
        break;
      default:
        help(argv[0]);
    }

    exit(0);
}

/* Revision History
 *
 * $Log: main.c,v $
 * Revision 1.11  2004/09/29 18:22:51  dechavez
 * corrected help message typo
 *
 * Revision 1.10  2004/04/26 21:22:41  dechavez
 * improved(?) help message, made uncompressed data the default
 *
 * Revision 1.9  2004/01/29 19:08:41  dechavez
 * use new isiFreeSoh(), isiFreeCnf(), isiFreeWfdisc()
 *
 * Revision 1.8  2003/12/23 00:12:40  dechavez
 * print out default in help message
 * allow for compile time override of default server define
 *
 * Revision 1.7  2003/12/11 22:27:10  dechavez
 * Changed data() to include building and freeing data request
 * Relink with libisi 1.4.7
 *
 * Revision 1.6  2003/11/21 20:27:26  dechavez
 * Changed default server to idahub (bad idea?) and corrected test for
 * isiReadGenericTS() failure cause
 *
 * Revision 1.5  2003/11/19 23:52:57  dechavez
 * print server name and operation when verbose option selected
 *
 * Revision 1.4  2003/11/19 21:35:55  dechavez
 * Rework to use simplified ISI API (libisi 1.4.0), and to include header in generic
 * data disk dumps
 *
 * Revision 1.3  2003/11/13 19:44:35  dechavez
 * Rework to use simplified ISI API (libisi 1.3.0)
 *
 * Revision 1.2  2003/11/04 00:03:24  dechavez
 * Rework to use ISI handle and related functions from version 1.1.0 of the library
 *
 * Revision 1.1  2003/10/16 18:05:23  dechavez
 * initial release
 *
 */
