#pragma ident "$Id: isi.h,v 1.13 2004/06/25 18:34:56 dechavez Exp $"
/*======================================================================
 *
 *  Ida System Interface library
 *
 *====================================================================*/
#ifndef isi_h_included
#define isi_h_included

#include "platform.h"
#include "iacp.h"
#include "list.h"

#ifdef __cplusplus
extern "C" {
#endif

/* IACP payload type codes (1000 - 1999) */

#define ISI_IACP_MIN          IACP_TYPE_ISI_MIN
#define ISI_IACP_REQ_SOH      ISI_IACP_MIN +  1 /* state of health request */
#define ISI_IACP_REQ_CNF      ISI_IACP_MIN +  2 /* configuration request */
#define ISI_IACP_REQ_WFDISC   ISI_IACP_MIN +  3 /* wfdisc request */
#define ISI_IACP_REQ_FORMAT   ISI_IACP_MIN +  4 /* format part of data request */
#define ISI_IACP_REQ_COMPRESS ISI_IACP_MIN +  5 /* compress part of data request */
#define ISI_IACP_REQ_POLICY   ISI_IACP_MIN +  6 /* policy part of data request */
#define ISI_IACP_REQ_STREAM   ISI_IACP_MIN +  7 /* stream part of data request */
#define ISI_IACP_SYSTEM_SOH   ISI_IACP_MIN +  8 /* system SOH data */
#define ISI_IACP_STREAM_SOH   ISI_IACP_MIN +  9 /* stream SOH data */
#define ISI_IACP_WFDISC       ISI_IACP_MIN + 10 /* wfdisc record */
#define ISI_IACP_STREAM_CNF   ISI_IACP_MIN + 11 /* configuration data */
#define ISI_IACP_GENERIC_TS   ISI_IACP_MIN + 12 /* ISI generic time series packet */
#define ISI_IACP_MAX          IACP_TYPE_ISI_MAX

/* Wildcards for time specifications */

#define ISI_UNDEFINED_TIMESTAMP ((REAL64) -1.0)
#define ISI_OLDEST              ((REAL64) -2.0)
#define ISI_NEWEST              ((REAL64) -3.0)
#define ISI_KEEPUP              ((REAL64) -4.0)

/* String lengths */

#define ISI_STALEN  7
#define ISI_CHNLEN  3
#define ISI_LOCLEN  2
#define ISI_OLDCHNLEN (ISI_CHNLEN+ISI_LOCLEN)
#define ISI_INAMLEN 7
#define ISI_STREAM_NAME_LEN (ISI_STALEN + 1 + ISI_CHNLEN + 1 + ISI_LOCLEN)

#define ISI_STA_CHN_LOC_DELIMITERS ".,:;/\\"

#define ISI_BLANK_LOC "  "

/* Reconnect policy codes */

#define ISI_RECONNECT_POLICY_UNDEF     0
#define ISI_RECONNECT_POLICY_NONE      1
#define ISI_RECONNECT_POLICY_MIN_GAP   2
#define ISI_RECONNECT_POLICY_MIN_DELAY 3

#define ISI_SHUTDOWN_DELAY 5000 /* 5 seconds between isiClose() and isiOpen() */

/* isiReadGenericTS return codes */

#define ISI_OK    0
#define ISI_DONE  1
#define ISI_ERROR 2
#define ISI_BREAK 3

/* Format codes */

#define ISI_FORMAT_UNDEF   0
#define ISI_FORMAT_GENERIC 1
#define ISI_FORMAT_NATIVE  2

/* misc data types */

typedef struct {
    char sta[ISI_STALEN+1];
    char chn[ISI_CHNLEN+1];
    char loc[ISI_LOCLEN+1];
    char oldchn[ISI_OLDCHNLEN+1];
} ISI_STREAM_NAME;

#define ISI_NAME_WILDCARD "*"

typedef struct {
    INT16 factor;
    INT16 multiplier;
    REAL64 value;
} ISI_SRATE;

typedef struct {
    REAL64 value;
    UINT16 status;
} ISI_TSTAMP;
#define ISI_TSTAMP_STATUS_UNKNOWN 0xffff

typedef struct {
    REAL32 lat;
    REAL32 lon;
    REAL32 elev;
    REAL32 depth;
} ISI_COORDS;

typedef struct {
    ISI_STREAM_NAME name; /* stream name */
    ISI_TSTAMP      tofs; /* time of oldest datum */
    ISI_TSTAMP      tols; /* time of youngest datum in disk loop */
    REAL64          tslw; /* time since last write to disk loop */
    UINT32          nseg; /* number of segments (gaps) in disk loop */
    UINT32          nrec; /* number of records received */
} ISI_STREAM_SOH;

typedef struct {
    REAL32  calib;
    REAL32  calper;
    REAL32  hang;
    REAL32  vang;
    char    type[ISI_INAMLEN+1];
} ISI_INST;

typedef struct {
    ISI_STREAM_NAME name;
    ISI_SRATE       srate;
    ISI_COORDS      coords;
    ISI_INST        inst;
} ISI_STREAM_CNF;

/* values for the ISI_STREAM_REQUEST status field */

#define ISI_COMPLETE   1
#define ISI_INCOMPLETE 2

typedef struct {
    ISI_STREAM_NAME name;
    REAL64 beg;
    REAL64 end;
    int status; /* internal client use only, never sent to server */
} ISI_STREAM_REQUEST;

typedef struct {
    UINT32 policy;
    UINT32 format;
    UINT32 compress;
    UINT32 nstream;
    ISI_STREAM_REQUEST *stream;
} ISI_DATA_REQUEST;

/* All the stuff needed to establish a connection on one handy structure */

typedef struct {
    int port;
    IACP_ATTR attr;
    int debug;
    struct {
        VOID(*func) (CHAR *string);
        char *spec;
        char *pname;
        BOOL enabled;
        LOGIO lp;
    } log;
    BOOL decompress;
} ISI_PARAM;

/* reports */

typedef struct {
    UINT32 nentry;
    ISI_STREAM_SOH *entry;
} ISI_SOH_REPORT;

typedef struct {
    UINT32 nentry;
    ISI_STREAM_CNF *entry;
} ISI_CNF_REPORT;

typedef struct {
    UINT32 nentry;
    char **entry;
} ISI_WFDISC_REPORT;

/* compression codes */

#define ISI_COMP_UNDEF  0
#define ISI_COMP_NONE   1
#define ISI_COMP_IDA    2
#define ISI_COMP_STEIM1 3
#define ISI_COMP_STEIM2 4

/* uncompressed waveform data type codes */

#define ISI_TYPE_UNDEF  0
#define ISI_TYPE_INT8   1
#define ISI_TYPE_INT16  2
#define ISI_TYPE_INT32  3
#define ISI_TYPE_INT64  4
#define ISI_TYPE_REAL32 5
#define ISI_TYPE_REAL64 6
#define ISI_TYPE_NATIVE 10
#define ISI_TYPE_IDA8   (ISI_TYPE_NATIVE + 1)
#define ISI_TYPE_IDA10  (ISI_TYPE_NATIVE + 2)
#define ISI_TYPE_IDA6   (ISI_TYPE_NATIVE + 3)
#define ISI_TYPE_IDA7   (ISI_TYPE_NATIVE + 4)

/* byte order codes */

#define ISI_ORDER_LTLENDIAN 1
#define ISI_ORDER_BIGENDIAN 0

/* datum description */

typedef struct {
    UINT8 comp;  /* ISI_COMP_x, defined above */
    UINT8 type;  /* ISI_TYPE_x, defined above */
    UINT8 order; /* ISI_ORDER_x, defined above */
    UINT8 size;  /* uncompressed data sample size in bytes */
} ISI_DATA_DESC;

/* channel status for compatibility with CD1.1 */

typedef struct {
    UINT8  data;
    UINT8  security;
    UINT8  misc;
    UINT8  voltage;
    REAL64 sync;
    UINT32 diff;
} ISI_CD_STATUS;

/* generic time series packet consists of header plus data */

typedef struct {
    ISI_STREAM_NAME name;   /* sta.chn.loc */
    ISI_SRATE       srate;  /* sample rate */
    ISI_TSTAMP      tofs;   /* time of first sample */
    ISI_TSTAMP      tols;   /* time of last sample */
    ISI_CD_STATUS   status; /* CD1.1 channel status data */
    UINT32          nsamp;  /* number of samples */
    ISI_DATA_DESC   desc;   /* datum description */
    UINT32          nbytes; /* number of data bytes */
} ISI_GENERIC_TSHDR;

typedef struct {
    BOOL DeletePossible;
    UINT32 AllocatedDataBytes;
    ISI_GENERIC_TSHDR hdr;
    VOID  *data;
} ISI_GENERIC_TS;

/* raw digitizer packets */

typedef struct {
    UINT32 type; /* packet type (IACP payload type code) */
    UINT32 len;  /* number of bytes in raw packet */
    UINT8  *hdr; /* points to start of raw packet header */
    UINT8  *dat; /* points to start of raw packet data */
    UINT8  *pkt; /* the entire raw data packet */
} ISI_RAW_PACKET;

/* handle for simplified interface */

typedef struct {
    UINT32 type;
    LIST   list;
    SEMAPHORE semaphore;
} ISI_INCOMING;

#define ISI_INTERNAL_BUFLEN 32768

typedef struct {
    char *server;
    int port;
    IACP_ATTR attr;
    LOGIO *lp;
    int debug;
    IACP *iacp;
    UINT8 buf[ISI_INTERNAL_BUFLEN];
    IACP_FRAME frame;
    ISI_DATA_REQUEST datreq;
    ISI_INCOMING incoming;
    ISI_GENERIC_TS *ts1;
    ISI_GENERIC_TS *ts2;
    BOOL decompress;
#define ISI_FLAG_NOP    0x00
#define ISI_FLAG_BREAK  0x01
    UINT8 flag;
} ISI;

/* Macros */

#define isiIsNative(desc) ((desc)->type > ISI_TYPE_NATIVE)
#define isiStaChnToStreamName(sta, chn, name) isiStaChnLocToStreamName(sta, chn, "  ", name)

#ifdef LTL_ENDIAN_HOST
#define ISI_HOST_BYTE_ORDER ISI_ORDER_LTLENDIAN
#else
#define ISI_HOST_BYTE_ORDER ISI_ORDER_BIGENDIAN
#endif

/* Defaults */

#define ISI_DEFAULT_PORT             39136
#define ISI_DEFAULT_HOST             "localhost"
#define ISI_DEFAULT_RECONNECT_POLICY ISI_RECONNECT_POLICY_MIN_GAP
#define ISI_DEFAULT_FORMAT           ISI_FORMAT_GENERIC
#define ISI_DEFAULT_STREAMSPEC       "*.*.*"
#define ISI_DEFAULT_COMP             ISI_COMP_IDA
#define ISI_DEFAULT_BEGTIME          ISI_NEWEST
#define ISI_DEFAULT_ENDTIME          ISI_KEEPUP

/* Function prototypes */

/* cnf.c */
char *isiStreamCnfString(ISI_STREAM_CNF *cnf, char *buf);
VOID isiPrintStreamCnf(FILE *fp, ISI_STREAM_CNF *cnf);
BOOL isiIacpSendStreamCnfList(IACP *iacp, LIST *list);
BOOL isiIacpRecvStreamCnfList(IACP *iacp, LIST *list);
LIST *isiRequestCnf(ISI *isi);

/* datreq.c */
VOID isiInitGenericTSHDR(ISI_GENERIC_TSHDR *src);
VOID isiPrintReqTime(FILE *fp, REAL64 value);
VOID isiPrintStreamReq(FILE *fp, ISI_STREAM_REQUEST *sreq);
VOID isiPrintDatreq(FILE *fp, ISI_DATA_REQUEST *dreq);
VOID isiSetDatreqPolicy(ISI_DATA_REQUEST *dreq, UINT32 value);
VOID isiSetDatreqFormat(ISI_DATA_REQUEST *dreq, UINT32 value);
VOID isiSetDatreqCompress(ISI_DATA_REQUEST *dreq, UINT32 value);
UINT32 isiGetDatreqCompress(ISI_DATA_REQUEST *dreq);
VOID isiInitDataRequest(ISI_DATA_REQUEST *req);
BOOL isiBuildStreamName(char *input, ISI_STREAM_NAME *out);
LIST *isiBuildStreamRequestList(REAL64 beg, REAL64 end, char *StreamSpec);
VOID isiClearDataRequest(ISI_DATA_REQUEST *req);
ISI_STREAM_REQUEST *isiLocateStreamRequest(ISI_STREAM_NAME *target, ISI_DATA_REQUEST *datreq);
BOOL isiIacpSendDataRequest(IACP *iacp, ISI_DATA_REQUEST *dreq);
UINT32 isiAppendStreamReq(ISI_INCOMING *incoming, IACP_FRAME *frame);
BOOL isiCopyStreamListToDataRequest(LIST *list, ISI_DATA_REQUEST *output);
BOOL isiSendDataRequest(ISI *isi, ISI_DATA_REQUEST *datreq, UINT32 *error);
ISI_DATA_REQUEST *isiFreeDataRequest(ISI_DATA_REQUEST *dreq);
ISI_DATA_REQUEST *isiAllocSimpleDataRequest(REAL64 beg, REAL64 end, char *spec);
ISI_DATA_REQUEST *isiAllocDefaultDataRequest(VOID);

/* ezio.c */
BOOL isiInitDefaultPar(ISI_PARAM *par);
ISI_PARAM *isiAllocDefaultParam(VOID);
VOID isiSetServerPort(ISI_PARAM *par, int port);
int isiGetServerPort(ISI_PARAM *par);
VOID isiSetTimeout(ISI_PARAM *par, int value);
VOID isiSetTcpBuflen(ISI_PARAM *par, int sndbuf, int rcvbuf);
VOID isiSetRetryFlag(ISI_PARAM *par, BOOL value);
VOID isiSetRetryInterval(ISI_PARAM *par, int value);
VOID isiSetDebugFlag(ISI_PARAM *par, int value);
BOOL isiSetLogging(ISI_PARAM *par, char *spec, VOID(*func) (char *string), char *pname);
ISI *isiInitiateDataRequest(char *server, ISI_PARAM *par, ISI_DATA_REQUEST *dreq);
ISI_SOH_REPORT *isiSoh(char *server, ISI_PARAM *par);
VOID isiFreeSoh(ISI_SOH_REPORT *report);
ISI_CNF_REPORT *isiCnf(char *server, ISI_PARAM *par);
VOID isiFreeCnf(ISI_CNF_REPORT *report);
ISI_WFDISC_REPORT *isiWfdisc(char *server, ISI_PARAM *par, int maxdur);
VOID isiFreeWfdisc(ISI_WFDISC_REPORT *report);


/* log.c */
VOID isiLogMsgLevel(ISI *isi, int level);
VOID isiLogMsg(ISI *isi, int level, char *format, ...);

/* open.c */
ISI *isiOpen(char *server, int port, IACP_ATTR *user_attr, LOGIO *lp, int debug);
ISI *isiFree(ISI *isi);
ISI *isiClose(ISI *isi);

/* pack.c */
int isiPackCoords(UINT8 *start, ISI_COORDS *src);
int isiUnpackCoords(UINT8 *start, ISI_COORDS *dest);
int isiPackInst(UINT8 *start, ISI_INST *src);
int isiUnpackInst(UINT8 *start, ISI_INST *dest);
int isiPackDatumDescription(UINT8 *start, ISI_DATA_DESC *src);
int isiUnpackDatumDescription(UINT8 *start, ISI_DATA_DESC *dest);
int isiPackSrate(UINT8 *start, ISI_SRATE *src);
int isiUnpackSrate(UINT8 *start, ISI_SRATE *dest);
int isiPackTstamp(UINT8 *start, ISI_TSTAMP *src);
int isiUnpackTstamp(UINT8 *start, ISI_TSTAMP *dest);
int isiPackStreamName(UINT8 *start, ISI_STREAM_NAME *src);
int isiUnpackStreamName(UINT8 *start, ISI_STREAM_NAME *dest);
int isiPackStreamRequest(UINT8 *start, ISI_STREAM_REQUEST *src);
int isiUnpackStreamRequest(UINT8 *start, ISI_STREAM_REQUEST *src);
int isiPackStreamCnf(UINT8 *start, ISI_STREAM_CNF *src);
int isiUnpackStreamCnf(UINT8 *start, ISI_STREAM_CNF *dest);
int isiPackStreamSoh(UINT8 *start, ISI_STREAM_SOH *src);
int isiUnpackStreamSoh(UINT8 *start, ISI_STREAM_SOH *dest);
int isiPackCdstatus(UINT8 *start, ISI_CD_STATUS *src);
int isiUnpackCdstatus(UINT8 *start, ISI_CD_STATUS *dest);
int isiPackGenericTSHDR(UINT8 *start, ISI_GENERIC_TSHDR *src);
int isiUnpackGenericTSHDR(UINT8 *start, ISI_GENERIC_TSHDR *dest);
int isiUnpackGenericTS(UINT8 *start, ISI_GENERIC_TS *dest);

/* soh.c */
char *isiStreamSohString(ISI_STREAM_SOH *soh, char *buf);
VOID isiPrintStreamSoh(FILE *fp, ISI_STREAM_SOH *soh);
BOOL isiIacpSendStreamSohList(IACP *iacp, LIST *list);
BOOL isiIacpRecvStreamSohList(IACP *iacp, LIST *list);
LIST *isiRequestSoh(ISI *isi);

/* string.c */
char *isiReqStatusString(int value);
char *isiFormatString(int value);
char *isiCompressString(int value);
char *isiPolicyString(int value);
char *isiStreamString(ISI_STREAM_NAME *name, char *buf);
char *isiTstampString(ISI_TSTAMP *tstamp, char *buf);
char *isiDescString(ISI_DATA_DESC *desc, char *buf);
char *isiGenericTsHdrString(ISI_GENERIC_TSHDR *hdr, char *buf);

/* ts.c */
BOOL isiInitGenericTS(ISI_GENERIC_TS *ts);
ISI_GENERIC_TS *isiCreateGenericTS(VOID);
VOID isiDestroyGenericTS(ISI_GENERIC_TS *ts);
BOOL isiCopyGenericTS(ISI_GENERIC_TS *src, ISI_GENERIC_TS *dest);
ISI_GENERIC_TS *isiReadGenericTS(ISI *isi, int *status);

/* utils.c */
ISI_SRATE *isiSintToSrate(REAL64 sint, ISI_SRATE *srate);
REAL64 isiSrateToSint(ISI_SRATE *srate);
int isiStreamNameCompare(ISI_STREAM_NAME *a, ISI_STREAM_NAME *b);
int isiStreamNameCompareWild(ISI_STREAM_NAME *a, ISI_STREAM_NAME *b);
BOOL isiStreamNameMatch(ISI_STREAM_NAME *a, ISI_STREAM_NAME *b);
VOID isiStaChnLocToStreamName(char *sta, char *chn, char *loc, ISI_STREAM_NAME *name);
VOID isiInitIncoming(ISI_INCOMING *incoming);
VOID isiResetIncoming(ISI_INCOMING *incoming);
VOID isiSetFlag(ISI *isi, UINT8 value);
UINT8 isiGetFlag(ISI *isi);

/* version.c */
char *isiVersionString(VOID);
VERSION *isiVersion(VOID);

/* wfdisc.c */
BOOL isiIacpSendWfdiscList(IACP *iacp, LIST *list);
BOOL isiIacpRecvWfdiscList(IACP *iacp, LIST *list);
LIST *isiRequestWfdisc(ISI *isi, int maxdur);

#ifdef __cplusplus
}
#endif

#endif

/* Revision History
 *
 * $Log: isi.h,v $
 * Revision 1.13  2004/06/25 18:34:56  dechavez
 * C++ compatibility
 *
 * Revision 1.12  2004/06/21 19:43:05  dechavez
 * defined types for ida rev's 6 and 7, blank loc constant and changed
 * isiStaChnToStreamName() macro to use blank loc instead of NULL
 *
 * Revision 1.11  2004/06/10 17:14:01  dechavez
 * added isiSetFlag() and isiGetFlag()
 *
 * Revision 1.10  2004/01/29 18:31:47  dechavez
 * added more prototypes
 *
 * Revision 1.9  2003/12/04 23:40:28  dechavez
 * renamed isiNativeByteOrder() macro with ISI_HOST_BYTE_ORDER
 *
 * Revision 1.8  2003/11/25 20:33:55  dechavez
 * added new prototypes
 *
 * Revision 1.7  2003/11/19 23:44:14  dechavez
 * updated prototypes, added isiStaChnToStreamName macro
 *
 * Revision 1.6  2003/11/19 21:23:51  dechavez
 * added bookkeeping hooks for buffer managment to ISI_GENERIC_TS, added
 * ISI_GENERIC_TS buffers to the ISI handle, added report structures
 *
 * Revision 1.5  2003/11/13 19:25:29  dechavez
 * updated prototypes
 *
 * Revision 1.4  2003/11/04 19:54:33  dechavez
 * added ISI_SHUTDOWN_DELAY, updated prototypes
 *
 * Revision 1.3  2003/11/04 00:39:02  dechavez
 * updated prototypes
 *
 * Revision 1.2  2003/11/03 23:08:52  dechavez
 * introduced ISI handle, ISI_PARAM structure, added additional prototypes,
 * consolidated defaults
 *
 * Revision 1.1  2003/10/16 15:38:40  dechavez
 * initial release
 *
 */
