#pragma ident "$Id: peer.c,v 1.5 2004/06/04 22:49:46 dechavez Exp $"
/*======================================================================
 *
 *  Get name and address of remote peer.
 *
 *====================================================================*/
#include "util.h"

char *util_peer(int sd, char *buffer, int buflen)
{
int addrlen;
#ifndef _WIN32
int h_errno;
#endif
struct sockaddr_in cli_addr;
struct hostent *hp, result;
static char *error = "<util_getpeer failed>";

    if (buflen < strlen(error)) return (char *) NULL;
    strcpy(buffer, error);

    addrlen = sizeof(cli_addr);
    cli_addr.sin_family = AF_INET;
    if (getpeername(sd, (struct sockaddr *)&cli_addr, &addrlen) != 0) {
        return (char *) NULL;
    }
#ifdef SOLARIS
    hp = gethostbyaddr_r( (char *) &cli_addr.sin_addr, sizeof(struct in_addr), cli_addr.sin_family, &result, buffer, MAXPATHLEN, &h_errno);
#else
    hp = gethostbyaddr(
        (char *) &cli_addr.sin_addr,
        sizeof(struct in_addr),
        cli_addr.sin_family
    );
#endif
    if (hp != NULL) {
        strcpy(buffer, hp->h_name);
    } else {
        strcpy(buffer, inet_ntoa(cli_addr.sin_addr));
    }

    return buffer;
}

char *utilPeerAddr(int sd, char *buffer, int buflen)
{
int addrlen;
char *dotdecimal;
struct sockaddr_in cli_addr;

    if (buflen < INET_ADDRSTRLEN) return NULL;
    addrlen = sizeof(cli_addr);
    if (getpeername(sd, (struct sockaddr *)&cli_addr, &addrlen) != 0) return NULL;

    dotdecimal = inet_ntoa(cli_addr.sin_addr);
    if (dotdecimal == NULL) return NULL;
    memcpy(buffer, dotdecimal, INET_ADDRSTRLEN);
    buffer[INET_ADDRSTRLEN-1] = 0;
#ifdef INET_NTOA_ALLOCATES_MEMORY
    free(dotdecimal);
#endif /* INET_NTOA_ALLOCATES_MEMORY */

    return buffer;
}

char *utilPeerName(int sd, char *buffer, int buflen)
{
int addrlen;
#ifndef _WIN32
int h_errno;
#endif;
struct sockaddr_in cli_addr;
struct hostent *hp, result;

    if (buflen < MAXPATHLEN) return NULL;

    addrlen = sizeof(cli_addr);
    cli_addr.sin_family = AF_INET;
    if (getpeername(sd, (struct sockaddr *)&cli_addr, &addrlen) != 0) return NULL;

#ifdef HAVE_GETHOSTBYADDR_R
    hp = gethostbyaddr_r(
        (char *) &cli_addr.sin_addr,
        sizeof(struct in_addr),
        cli_addr.sin_family,
        &result,
        buffer,
        buflen,
        &h_errno
    );
#else
    hp = gethostbyaddr(
        (char *) &cli_addr.sin_addr,
        sizeof(struct in_addr),
        cli_addr.sin_family
    );
#endif

    if (hp != NULL) {
        strcpy(buffer, hp->h_name);
    } else {
        utilPeerAddr(sd, buffer, buflen);
    }

    return buffer;
}

int utilPeerPort(int sd)
{
int addrlen;
struct sockaddr_in cli_addr;

    addrlen = sizeof(cli_addr);
    if (getpeername(sd, (struct sockaddr *)&cli_addr, &addrlen) != 0) return -1;

    return (int) ntohs(cli_addr.sin_port);
}


/* Revision History
 *
 * $Log: peer.c,v $
 * Revision 1.5  2004/06/04 22:49:46  dechavez
 * various AAP windows portability modifications
 *
 * Revision 1.4  2004/01/29 18:23:02  dechavez
 * don't free inet_ntoa() return value in utilPeerAddr().  inet_ntoa is
 * not reentrant
 *
 * Revision 1.3  2003/06/09 23:59:27  dechavez
 * added utilPeerAddr(), utilPeerName(), utilPeerPort()
 *
 * Revision 1.2  2001/10/24 23:27:44  dec
 * use cli_addr. instead of cli_addrp->
 *
 * Revision 1.1.1.1  2000/02/08 20:20:41  dec
 * import existing IDA/NRTS sources
 *
 */
