import  sys, os, time, signal
from    getopt import getopt, GetoptError
import  seis, seiscomp, seiscomp.slclient, seiscomp.detector
from    seiscomp.detector import MyDetector as Detector, TriggerConfig, \
                                 detector_matches_record
from    seiscomp.myconfig import MyConfig
from    seis.msg import Logger

def usage(exitcode=0):

    seis.message("""
    autopick.py - The SeisComP automatic picking program

    Usage: autopick.py [options]

    Options:
    -c, --config f          config file (default: config.ini)
    -t, --trigger-config f  trigger config file (default: trigger.ini)
    -f, --file f            file to read input data from
    -h, --help              show usage message
    """)
    sys.exit(exitcode)

#   -b, --before t          start analysis t hours before now
#   -s, --server s          SeedLink server (default: localhost:18000)

ifile = sys.stdin
detec = dict()
test = False
verbosity = 0
filename = None
before = None
server = None
trigger_ini = "trigger.ini"
db_config = None
logdir = None

try:
    opts, args = getopt(sys.argv[1:], "b:c:d:f:l:s:t:hv",
        [   "before=", "config=", "logdir=", "db-config=", "file=", "server=", "detec-id=",
            "verbose", "help"])
except GetoptError:
    usage(exitcode=2)

for flag, arg in opts:
    if flag in ("-b", "--before"):
        before = float(arg)
    if flag in ("-i", "--fifo"):
        raise NotImplementedError, "FIFO not yet implemented"
    elif flag in ("-c", "--config"):
        config_ini = arg
    elif flag in ("-d", "--db-config"):
        db_config = arg
    elif flag in ("-t", "--trigger-config"):
        trigger_ini = arg
    elif flag in ("-l", "--logdir"):
        logdir = arg
    elif flag in ("-f", "--file"):
        filename = arg
    elif flag in ("-s", "--server"):
        server = arg
    elif flag == "--detec-id":
        pass # this is a dummy arg used to identify the process
    elif flag in ("-h", "--help"):
        usage(exitcode=0)
    elif flag in ("-v", "--verbose"):
        verbosity += 1

if logdir:
        logger = Logger(os.path.join(logdir, "autopick-log-"))
else:   logger = sys.stderr
    
seiscomp.detector.verbose = verbosity # XXX ugly...

if db_config:
    import seiscomp.event_db
    if verbosity >= 2:
        logger.write("reading db config from file '%s'\n" % db_config)
    db_config = seiscomp.event_db.get_config(db_config)
    if verbosity >=3:
        seis.message(str(dbconfig).strip())
    db = seiscomp.event_db.EventDB(db_config)
else:
    db = None

def sig_handler(signum, frame):
    global input
    del    input
                
signal.signal(signal.SIGHUP, sig_handler)

config = MyConfig(config_ini)
if not server: # if 'server' not specified, use value from config.ini
    server = config['config']['server']
if not before: # if 'before' not specified, use value from config.ini
    before = float(config['config']['before'])

# determine the last motification time of the trigger.ini file
trigger_ini_mtime = os.stat(trigger_ini)[8]

trigger_config = TriggerConfig(trigger_ini)

for label in trigger_config:
    if verbosity: logger.write("creating detector '%s'\n" % label)
    detec[label] = Detector(label, trigger_config[label], logger, db)

if filename:    # read from file
    if filename == "-":
        ifile = sys.stdin
    else:
        ifile = file(filename)

    input = seiscomp.mseed.Input(ifile)

else:           # connect to SeedLink server
    streams = [(d.net,d.sta,d.loc,d.cha) for d in detec.values()]

    if before:
            stime = seis.time() - before*3600
    else:   stime = None

    input = seiscomp.slclient.Input(server, streams, stime)


for rec in input:

    for label in detec: # search for matching detectors
        det = detec[label]
        if detector_matches_record(det, rec):
            if verbosity>=4:
                seis.message("feeding %s to %s" % (rec.stream_id, label))
            try:
                det.feed(rec)
            except OverflowError:
                msg  = "OverflowError\n"
                msg += "  Detector %s\n" % label
                msg += "  Stream   %s\n" % rec.stream_id
                msg += "  Time     %s\n" % rec.time
                logger.write(msg)
