import sys, os, time

def message(msg):
    """ Write a message to stderr. A newline is appended. """

    sys.stderr.write("%s\n" % msg)


def warning(msg):
    """ Write a warning to stderr. A newline is appended. """

    message("Warning: %s" % msg)


def error(msg, exitcode=1, prefix="ERROR:"):
    """
    Write a error message to stderr and exit. Each line is prefixed
    and a newline is appended.
    """

    lines = msg.rstrip("\n").split("\n")
    for line in lines:
        message("%s %s" % (prefix, line))


class Logger(object):

    def __init__(self, path_prefix, date_prefix=False):
        self.path_prefix = path_prefix
        self.date_prefix = date_prefix
        self.last        = None
        self.file        = None

#   def __call__(self, msg):
#       self.write(msg)

    def write(self, msg, now=None):
        if now is None:
            now = time.localtime()
        date = time.strftime("%y%m%d", now)
        if date!=self.last:
            new = self.path_prefix + date
            lnk = self.path_prefix + "current"
            self.file = file(new, "a")
            if os.path.exists(lnk):
                if os.readlink(lnk) != new:
                    os.remove(lnk)
            if not os.path.exists(lnk):
                os.symlink(new, lnk)
            self.last = date
        if self.date_prefix:
            date = time.strftime("%y%m%d %H%M%S", now)
            msg = "%s: %s" % (date, msg)
        self.file.write("%s" % msg)
        self.file.flush()

    def flush(self):
        self.file.flush()


def logger(path_prefix="/tmp/log-"):
    return Logger(path_prefix)
