def which(program):
    """
    Returns the complete path to 'program'. The functionality is
    equivalent to that of the GNU tool 'which' or similar to the
    'type -p' builtin of some Unix shells.

    The program must reside in a directory pointed to by the PATH
    environment variable; otherwise it cannot be found.

    Tested under Linux and should work under any POSIX OS.
    """

    import os
    assert os.name == "posix"

    result = None

    try:
        path_list = os.environ["PATH"]
    except KeyError:
        path_list = os.defpath
    path_list = path_list.strip(os.pathsep).split(os.pathsep)

    for dir in path_list:
        p = os.path.join(dir, program)
        try:
            s = os.stat(p)
        except OSError:
            continue

        if not os.access(p, os.X_OK):
            continue

        result = p
        break

    return result


def commented_file(fname, commchar='#'):

    """
    Iterates over the lines of a text file stripping out
    comments starting with commchar
    """ 

    f = file(fname)
    for line in f.xreadlines():
        line = line.strip()
        pos = line.find(commchar)
        if pos != -1:
            line = line[:pos]
        if len(line) == 0:
            continue

        yield line

    f.close()

