#db_type = "sqlite" # XXX THIS IS ONLY FOR TESTING, IT DOESN'T WORK YET  XXX
db_type = "mysql"
#db_type = "postgresql"

import sys
import seis

if db_type == "mysql":
    from MySQLdb \
    import Connect as ConnectMySQL, OperationalError, paramstyle
elif db_type == "sqlite":
    from pysqlite2.dbapi2 \
    import connect as ConnectSQLite, OperationalError, paramstyle
else:
    sys.exit(10)

if paramstyle == "format":
    xparamstyle = "%s"
elif paramstyle == "qmark":
    xparamstyle = "?"
else:
    raise ImportError, "unable to handle paramstyle=\"%s\"" % paramstyle

def mysql_errno(exception):
    msg = str(exception)
    assert msg[0]=='(' and msg[5]==','
    return int(msg[1:5])

class ConfigMySQL:

    def __init__(self, host=None, port=None, user=None, password=None,
                       dbname=None):
        self.mysql_host =   host
        self.mysql_port =   port
        self.mysql_user =   user
        self.mysql_pass =   password
        self.mysql_dbname = dbname

    def __str__(self):
        x = "host = %s\n" % self.mysql_host
        x+= "port = %s\n" % self.mysql_port
        x+= "user = %s\n" % self.mysql_user
        x+= "pass = %s\n" % (len(self.mysql_pass)*'*')
        x+= "db   = %s\n" % self.mysql_dbname
        return x

class ConfigSQLite:

    def __init__(self, path):
        self.path = path

class ConfigPostgreSQL:
    pass

if db_type == "mysql":
    Config = ConfigMySQL
elif db_type == "sqlite":
    Config = ConfigSQLite

def get_config(explicit_filename=None):

    import os

    filename = explicit_filename

    if not filename:
        filename = os.getenv("SEISCOMP_DB_CONF")

    if not filename:
        raise LookupError, "Cannot find SeisComP DB configuration file"

    f = file(filename)

    if db_type == "mysql":
        xx = [ [ y.strip() for y in x.split("=", 1) ]
                    for x in f.readlines() if x[:2].isalpha()]
        d = {}
        for x,y in xx: d[x] = y
        return ConfigMySQL(host=d["host"], port=int(d["port"]),
                      user=d["user"], password=d["pass"],
                      dbname=d["db"])
                      
    elif db_type == "sqlite":
        return ConfigSQLite("/home/saul/seiscomp.db")


def errorexit(msg):
    sys.stderr.write("%s\n" % msg)
    sys.exit(1)


class _MyCursor(object):

    # The purpose of this is to automatically detect a lost connection and
    # consequently to reconnect to the server

    def __init__(self, db):
        self.db  = db
#       self.cur = db.con.cursor()
        self.cur = db.cur

    def execute(self, query, args=None):
        if args:
                r = query % self.cur.connection.literal(args)
        else:   r = query
        self.db.debug("%s\n" % r)

        while True:
            try:
                if args is None: # This distinction is required by pysqlite2
                        self.cur.execute(query)
                else:   self.cur.execute(query, args)
                self.db.con.commit()
                return
            except OperationalError,e: # apparently connection got lost
                print e
                if db_type == "mysql":
                    errno = mysql_errno(e)
                    if errno!=2006:
                        raise OperationalError, e
                # at this point it is assumed that the connection was lost
                # and that a reconnect() is required
                self.db.reconnect()
                self.cur = self.db.cur

    def fetchall(self):

        while True:
            try:
                return self.cur.fetchall()
            except OperationalError: # apparently connection got lost
                self.db.reconnect()
                self.cur = db.cur


class _MyConnection(object):

    def __init__(self, *conn_args):
        self.__conn = MySQLdb.connect(*conn_args)
        self.__conn_args = conn_args

    def cursor(self):
        return MyCursor(self.__conn, self.__conn_args)


class _MyDB(object):

    try_connect_max = 1000
    try_connect_dt  = 2

    def __init__(self, config, logger=sys.stderr, verbosity=0):

        if paramstyle not in ("format", "qmark"):
            raise Error, "unsupported paramtyle '%s'" % paramstyle # FIXME

        self.agency    = "GFZ"
        self.config    = config
        self.logger    = logger
        self.verbosity = verbosity
        self.connect()

    def connect(self):
        """ Try to connect to server """

        self.debug("Connecting to %s database\n" % db_type)

        if db_type == "mysql":
            self.con = ConnectMySQL(host   = self.config.mysql_host,
                                    port   = self.config.mysql_port,
                                    user   = self.config.mysql_user,
                                    passwd = self.config.mysql_pass,
                                    db     = self.config.mysql_dbname)

        elif db_type == "sqlite":
            self.con = ConnectSQLite("/home/saul/seiscomp.db")

        self.cur = self.con.cursor()

    def cursor(self):
        return _MyCursor(self)
#       return self.cur

    def reconnect(self): 
        # Try to reconnect a few times
        self.logger.write("%s lost connection to db server - " \
                          "trying to reconnect\n" % str(seis.time()))
        for attempt in xrange(self.try_connect_max):
            try:
                return self.connect()
            except OperationalError:
                self.logger.write("failed to reconnect - retrying\n")
                import time
                time.sleep(self.try_connect_dt)
    
        raise OperationalError, "failed to reconnect - giving up"

    def disconnect(self):
        pass

    def error(self, msg):
        self.logger.write(msg)

    def write(self, msg):
        self.logger.write(msg)

    def verbose(self, msg):
        if self.verbosity>=1:
            self.logger.write(msg)

    message = verbose

    def debug(self, msg):
        if self.verbosity>=2:
            self.logger.write(msg)
