import sys, time, md5
from Numeric import average, log, exp, sqrt, sum
from MLab    import median
from seiscomp.autoloc_magnitude  import lmagn, bmagn
from seiscomp.autoloc_misc      import sorted_by_time
from seiscomp._datatypes import Origin
from seiscomp.fe import region

testmode=True

def print_origin_autoloc1(org):
    # Prints an origin in "old" AutoLoc1 format
    # XXX perhaps make it a member of locsat.Origin
    reg = region(org.lat, org.lon)

    # determine the magnitudes and store them in dictionaries.

    mb, amp_mb, per_mb = compute_magnitude_mb(org)
    ml, amp_ml         = compute_magnitude_ml(org)

    mb_nst = ml_nst = 0
    mb_avg = ml_avg = 0.

    try:
        _mlist = [m for m in mb.values() if m != 0.]
        mb_avg = average(_mlist)
        mb_nst = len(_mlist)
    except ZeroDivisionError: # no valid magnitude
        pass

    try:
        _mlist = [m for m in ml.values() if m != 0.]
        ml_avg = average(_mlist)
        ml_nst = len(_mlist)
    except ZeroDivisionError: # no valid magnitude
        pass

    mag_avg, mag_name = mb_avg, "mb"
    if ml_nst > mb_nst: mag_avg, mag_name = ml_avg, "ML"

    if testmode:
            arr = org.arr
    else:   arr  = [ a for a in org.arr if a.used ]

    arr  = sorted_by_time(arr)
    nsta = len(arr)

    msg = ""

    msg += " Autoloc alert %s: determined by %s stations\n\n" % (org.id, nsta)
    msg += " LocSAT solution (with start solution, %d stations used, weight %d):\n\n" % (nsta, nsta)
    ns, ew = "NS"[org.lat<0], "EW"[org.lon<0]
    tim = "  ".join(str(org.time)[:-4].split())
    msg += "  %s  %s=%.1f  %s  %.2f %c  %.2f %c  %3.0f km\n\n" % \
      (reg, mag_name, mag_avg, tim, abs(org.lat), ns, abs(org.lon), ew, org.dep)
    msg += "  Stat  Net   Date       Time           Amp    Per   Res  Dist   Az   mb   ML\n"

    for a in arr:
        try:    _mb, amb, pmb = mb[a.sta], amp_mb[a.sta], per_mb[a.sta]
        except: _mb = amb = pmb = 0.

        try:    _ml, aml = ml[a.sta], amp_ml[a.sta]
        except: _ml = aml = 0.
        _tx = str(a.time)

        if testmode:
            if a.used:  flag="*"
            else:       flag="-"
        else: flag = " "

        msg += "  %-5s %2s  %s  %s     %8.1f  %3.1f %5.1f %5.1f %5.1f %4.1f %4.1f\n" % (a.sta,a.net, _tx[2:10], _tx[11:-2], amb, pmb, a.res, a.dist, a.azi, _mb,_ml)
#       msg += "%c %-5s %2s  %s  %s     %8.1f  %3.1f %5.1f %5.1f %5.1f %4.1f %4.1f\n" % (flag, a.sta,a.net, _tx[2:10], _tx[11:-2], amb, pmb, a.res, a.dist, a.azi, _mb,_ml)
    arr  = [ a for a in org.arr if not a.used ]
    for a in arr:
        msg += "x %-5s %2s %s     %8.1f  %3.1f %5.1f %5.1f %3.0f\n" % (a.sta,a.net,str(a.time)[:-2], amb, pmb, a.res, a.dist, a.azi)
    return msg

def print_origin_procalert(org, f=sys.stdout):
    # Prints an origin in the new AutoLoc2 format
    # Still undergoing frequent changes!

    org.compute_magnitudes()

    msg = ""

    ns, ew = "NS"[org.lat<0], "EW"[org.lon<0]
    msg += " Event-ID:    %s\n\n" % org.id
    _date, _time = str(org.time)[:-2].split()

    str1 = org.time.strftime("%d %B %Y   (%j)")
    str2 = region(org.lat, org.lon)
    msg += " %-36s  %-40s\n\n" % (str1, str2)

    msg += " Location estimate (+/- standard deviation)\n"
    if org.time_err is None:
        msg += "  Origin time       %s\n" % _time
        msg += "  Latitude          %6.2f %c\n" % (abs(org.lat), ns)
        msg += "  Longitude         %6.2f %c\n" % (abs(org.lon), ew)
        msg += "  Depth             %5.1f  km\n" % org.dep
    else:
        msg += "  Origin time       %s  +/- %5.1f sec\n" % (_time, org.time_err)
        msg += "  Latitude          %6.2f %c    +/- %5.1f km\n" \
                % (abs(org.lat), ns, org.lat_err)
        msg += "  Longitude         %6.2f %c    +/- %5.1f km\n" \
                % (abs(org.lon), ew, org.lon_err)
        if org.dep_err > 0.01:
                dep_err = "+/- %5.1f km" % org.dep_err
        else:   dep_err = "(fixed)"
        msg += "  Depth             %5.1f  km   %s\n" % (org.dep, dep_err)
    
    msg += "\n Magnitudes:\n"
    msg += "  ML=%.1f (%d)  mb=%.1f (%d)\n" \
        % (org.ml_avg, org.ml_nst, org.mb_avg, org.mb_nst)
    msg += "  Median=%.1f (%d)\n" % (org.m_med, org.m_nst)
    msg += "\n"
    msg += " 90-Percent confidence region ellipse:\n" \
            "  Semi-major axis   %6.1f km  strike: %.1f deg\n" \
            "  Semi-minor axis   %6.1f km\n" \
            "  Origin time error %6.1f sec\n\n" \
            % (org.confreg_semmaj, org.confreg_majaxi,
               org.confreg_semmin, org.confreg_timerr)
    tmp =  print_origin_autoloc1(org)
    org.md5hash = md5.new(tmp).digest()
    msg += tmp
    msg += "\n"
    msg += " RMS-ERR:         %.2f\n\n" % org.rms_res
    msg += " First location:  %s\n"     % time.strftime("%Y/%m/%d %T UTC", org.detected) 
    msg += " This location:   %s\n"     % time.strftime("%Y/%m/%d %T UTC", time.gmtime()) 

    return msg


def print_origin_procalert_old(org, f=sys.stdout):
    # Prints an origin in the new AutoLoc2 format
    # Still undergoing frequent changes!

    # determine the magnitudes and store them in dictionaries.
    mb, amp_mb, per_mb = compute_magnitude_mb(org)
    ml, amp_ml         = compute_magnitude_ml(org)
    try:
        _mlist = [m for m in mb.values() if m != 0.]
        mb_avg = average(_mlist)
        mb_nst = len(_mlist)
    except ZeroDivisionError: # no valid magnitude
        mb_avg, mb_nst = 0., 0
    try:
        _mlist = [m for m in ml.values() if m != 0.]
        ml_avg = average(_mlist)
        ml_nst = len(_mlist)
    except ZeroDivisionError: # no valid magnitude
        ml_avg, ml_nst = 0., 0

    msg = ""

    ns, ew = "NS"[org.lat<0], "EW"[org.lon<0]
    msg += " Event-ID:    %s\n\n" % org.id
    _date, _time = str(org.time)[:-2].split()

    str1 = org.time.strftime("%d %B %Y   (%j)")
    str2 = region(org.lat, org.lon)
    msg += " %-36s  %-40s\n\n" % (str1, str2)

    msg += " Location estimate (+/- standard deviation)\n"
    msg += "  Origin time       %s  +/- %5.1f sec\n" \
                % (_time, org.time_err)
    msg += "  Latitude          %6.2f %c    +/- %5.1f km\n"\
                % (abs(org.lat), ns, org.lat_err)
    msg += "  Longitude         %6.2f %c    +/- %5.1f km\n"\
                % (abs(org.lon), ew, org.lon_err)
    if org.dep_err > 0.01:
            dep_err = "+/- %5.1f km" % org.dep_err
    else:   dep_err = "(fixed)"
    msg += "  Depth             %5.1f  km   %s\n" % (org.dep, dep_err)
    msg += "\n Magnitudes:\n"
    mb,nmb,ml,nml=0.,0,0.,0
    msg += "  ML=%.1f (%d)  mb=%.1f (%d)\n" % (ml_avg, ml_nst, mb_avg, mb_nst)
    msg += "\n"
    msg += " 90-Percent confidence region ellipse:\n" \
            "  Semi-major axis   %6.1f km  strike: %.1f deg\n" \
            "  Semi-minor axis   %6.1f km\n" \
            "  Origin time error %6.1f sec\n\n" \
            % (org.confreg_semmaj, org.confreg_majaxi,
               org.confreg_semmin, org.confreg_timerr)
    msg += print_origin_autoloc1(org)
    msg += "\n"
    msg += " RMS-ERR:         %.2f\n\n" % org.rms_res
    msg += " First location:  %s\n"     % time.strftime("%Y/%m/%d %T UTC", org.detected)
    msg += " This location:   %s\n"     % time.strftime("%Y/%m/%d %T UTC", time.gmtime())

    f.write(msg)
    f.flush()

def print_origin_seismic_handler(org, f=sys.stdout):

    arr   = [ a for a in org.arr if a.used ]
    arr   = sorted_by_time(arr)
    evtnr = int(org.time.time1970 % 86400)/90
    evtid = "%03d%02d%02d%03d" % \
                (org.time.yr%1900, org.time.mo, org.time.day, evtnr)
    msg = ""
    for a in arr:
        tstr = a.time.strftime("%d-%b-%Y_")+str(a.time).split()[1]
        msg += "Event ID               : %s\n" % evtid
        msg += "Station code           : %s\n" % a.sta
        msg += "Onset time             : %s\n" % tstr.upper()
        msg += "Onset type             : emergent\n"
        msg += "Phase name             : P\n"
        msg += "Component              : Z\n"
        msg += "Quality number         : 2\n"
        msg += "--- End of Phase ---\n\n\n"

    tstr = org.time.strftime("%d-%b-%Y_")+str(org.time).split()[1]
    msg += "Depth (km)             : %.2f\n" % org.dep
    msg += "Latitude               : %.2f\n" % org.lat
    msg += "Longitude              : %.2f\n" % org.lon
    msg += "Depth type             : ( ) free\n"
    msg += "Origin time            : %s\n" % tstr.upper()
    msg += "Region Table           : Flinn-Engdahl\n"
    msg += "Region ID              : 0\n"
    msg += "Source region          : %s\n" % region(org.lat, org.lon)
    msg += "Location method        : locsat\n"
    msg += "Reference Location Name: RUE\n"
    msg += "Source of Information  : GFZ\n"
    msg += "Error in Depth (km)    : 0.00\n"
    msg += "Phase Flags            : L\n"
    msg += "--- End of Phase ---\n"

    f.write(msg)


def print_origin_sortlist(org, f=sys.stdout):
    # Prints the triggers contributing to an origin in the
    # "old" AutoLoc sortlist format
    tmp = [ (trg.on, trg) for trg in org.trgs if trg.used ]
    tmp.sort()
    for t, trg in tmp:
        # this must clearly be replaced by something more elegant
        f.write("%s 000\n" % trg.print_autoloc()[:-4])

def print_summary_line(org):
    return "%s %2s %s %7.2f %7.2f %3d %4.2f %s\n" % (org.id, org.status,
    org.time,org.lat,org.lon,int(org.dep), org.rms_res, " ".join([a.sta for a
    in org.arr if a.used]))

#def print_summary_line(org, f=sys.stdout):
#    f.write("%s %2s %s %7.2f %7.2f %3d %4.2f %s\n" % (org.id, org.status,
#    org.time,org.lat,org.lon,int(org.dep), org.rms_res, " ".join([a.sta for a
#    in org.arr])))
#    f.flush()


class Origin(object):
    __slots__ = ["detected","bad","status","id",
        "lat", "lat_err", "lon", "lon_err", "dep", "dep_err",
        "time", "time_err", "rms_res", "confreg_semmaj", "confreg_semmin",
        "associations",
        "confreg_majaxi", "confreg_timerr", "trgs", "arr",
        "mag_mb", "amp_mb", "per_mb", "mag_ml", "amp_ml",
        "mb_avg", "mb_nst", "ml_avg", "ml_nst", "m_med", "m_nst","md5hash" ]

#   __slots__ = ["detected","bad","status","id",
#       "lat", "lat_err", "lon", "lon_err", "dep", "dep_err",
#       "time", "time_err", "rms_res", "confreg_semmaj", "confreg_semmin",
#                 "confreg_majaxi", "confreg_timerr","trgs","arr" ]

    def __init__(self):
#       for attr in self.__slots__:
#           setattr(self, attr, 0.)
        self.id = ""
        self.time = None
        self.confreg_timerr = self.confreg_semmaj = \
            self.confreg_semmin = self.confreg_majaxi = 0. # XXX
        self.time_err = self.lat_err = self.lon_err = self.dep_err = 0. # XXX
        self.rms_res = 0 # XXX
        self.detected = time.gmtime() # XXX
        self.status = "N"

    def remove_arrival(self, id):
        self.trgs = [ trg for trg in self.trgs if trg.id != id ]
        self.arr  = [ arr for arr in self.arr  if arr.id != id ]

    def compute_magnitudes(self):
        # determine the magnitudes mb and ml and store them in dictionaries.
        _mlist_all = []
        self.mag_mb, self.amp_mb, self.per_mb = compute_magnitude_mb(self)
        self.mag_ml, self.amp_ml              = compute_magnitude_ml(self)

        _mlist = [m for m in self.mag_mb.values() if m != 0.]
        _mlist_all.extend(_mlist)
        self.mb_nst = len(_mlist)
        if self.mb_nst == 0:
                self.mb_avg = 0.
        else:   self.mb_avg = average(_mlist)

        _mlist = [m for m in self.mag_ml.values() if m != 0.]
        _mlist_all.extend(_mlist)
        self.ml_nst = len(_mlist)
        if self.ml_nst == 0:
                self.ml_avg = 0.
        else:   self.ml_avg = average(_mlist)

        self.m_nst = len(_mlist_all)
        if self.m_nst == 0:
                self.m_med = 0.
        else:   self.m_med = median(_mlist_all)

    def compute_residuals(self):
        raise NotImplementedError

class LocationFailed(Exception):

    def __init__(self, msg, exception=None):
        self._msg = msg
        self._exception = exception
        Exception.__init__(self, msg)

    def getMessage(self):
        return self._msg

    def getException(self):
        return self._exception

    def __str__(self):
        return self._msg

    def __getitem__(self, ix):
        """ Avoids weird error messages if someone does exception[ix]
            by mistake, since Exception has __getitem__ defined."""
        raise AttributeError("__getitem__")


def sort_trial_origins(trial_origins):
    
    new_trial_origins = []
    for org in trial_origins: 
        amp_wt = sum([ log(trg.ampl_max) for trg in org.trgs ])
        new_trial_origins.append( (len(org.trgs), amp_wt, org) ) 
#       new_trial_origins.append( (len(org.trgs), -org.rms_res, org) ) 

    new_trial_origins.sort()
    new_trial_origins.reverse()

    trial_origins = []
    for n, x, org in new_trial_origins:
        trial_origins.append( org )

    return trial_origins


def remove_redundant_trial_origins(trial_origins):
    # often the same set of triggers gets associated to different beams,
    # but we don't want to try a locating for each of these. This is tested
    # only based on station names.
    new_trial_origins = []
    sta_trial_origins = []

    for org in trial_origins: # XXX
        stations = [ trg.sta for trg in org.trgs ]
        stations.sort()
        stations = ':'.join(stations)
        stations += str(org.dep)
        if stations in sta_trial_origins:
            continue
        sta_trial_origins.append(stations)
        new_trial_origins.append(org)

    return new_trial_origins


def origin_update(org, new_org):
    for attr in new_org.__slots__:
        if attr in [ "id", "status" ]:
            continue
        try:
            setattr(org, attr, getattr(new_org, attr))
        except: continue
    org.status = "U"


def good_origin(org):
    # test if a origin is "good", i.e. we shall add it to our list of
    # properly located events
    if len(org.trgs) > 15:
        return True
    return False


def rms_penalty(rms_value, base_value=1.5, x=4.):
    # This is an empirical function meant to "flatten" the RMS for values less
    # than 'base_value', which are considered more likely to be coincidental
    # rather than indicative of a "better" origin.
    #
    # XXX A distance-dependent function might be more appropriate.

    return (rms_value**x+base_value**x)**(1./x)


def reg_penalty(maj_axis, base_value=30., x=3.):
    return (maj_axis**x+base_value**x)**(1./x)


def compare_origin_quality(org1, org2, logger=None):
    # very subjective "comparison" between two event origins
    #
    # returns a number > 0 if org2 is "better" than org1, 0 if they are
    # "equally good" and < 1 otherwise.
    #
    # All sorts of criteria may be formulated here. The question is:
    # "Is org2 so much better than org1 that we can safely replace
    #  org1 by org2?"

    x = 0.

    # test if the origins are actually equivalent...
    id1 = [ trg.id for trg in org1.trgs ]
    id2 = [ trg.id for trg in org2.trgs ]
    id1.sort(); id1 = ':'.join(id1)
    id2.sort(); id2 = ':'.join(id2)
#   if id1==id2:
        # if the trigger set is the same, we compare the RMS residuals (since
        # the origins may be different, particularly due to different fixed
        # focal depths
    # The relative weight of the RMS residuals should be weighted _low_
    # (0.3) compared to the size of the confidence region (result from testing)
    dx = 0.3*log(rms_penalty(org1.rms_res)/rms_penalty(org2.rms_res))
    if logger:
        logger.write("compare_origin_quality rms %.3f %.3f\n" \
                     "compare_origin_quality rms dx= %.3f\n" % \
                      (org1.rms_res, org2.rms_res, dx))
    x += dx

    # **** Distances from epicenter ****
    # This incorporates number of triggers, but weights closer stations higher
    # XXX Maybe replace this by a function of the actualy phase slowness
    xx1 = sum([(1./sqrt(a.dist+18.)) for a in org1.arr if a.used])
    xx2 = sum([(1./sqrt(a.dist+18.)) for a in org2.arr if a.used])
    dx = 0.6*(xx2-xx1) # or make it logarithmic?
    x += dx
    
#       if org1.rms_res < 0.98*org2.rms_res: return -1
#       if org2.rms_res < 0.98*org1.rms_res: return  1
#       return 0

#   x = 0.
#   dx = 4*log(len(org2.trgs)*1./len(org1.trgs))
### dx = 0.4*(len(org2.trgs)-len(org1.trgs)) # don't make it logarithmic
    if logger:
        logger.write("compare_origin_quality ntr A %s\n" % \
            " ".join([t.sta for t in org1.arr if a.used]))
        logger.write("compare_origin_quality ntr B %s\n" % \
            " ".join([t.sta for t in org2.arr if a.used]))
        logger.write("compare_origin_quality ntr+dist dx= %.3f\n" % dx)
### x += dx

    # XXX
#   if len(org2.trgs) > len(org1.trgs):
#       x += 1

    try:
        conf1 = reg_penalty(org1.confreg_semmaj)
        conf2 = reg_penalty(org2.confreg_semmaj)
        dx = 0.3*log(conf1/conf2)
        x += dx
        if logger:
            logger.write("compare_origin_quality reg %.3f %.3f\n" % \
                (conf1, conf2))
            logger.write("compare_origin_quality reg dx= %.3f\n" % dx)
    except ZeroDivisionError:
        if logger:
            logger.write("compare_origin_quality reg failed")

    # perhaps add further tests here
#   if x<0: return -1
#   if x>0: return  1
#   return  0
    return x

def dist_wt(x):
    # attempt to downweight residuals for stations close to epicenter
    return 1./(1.+exp(-x*x/50.))


def check_consistent_magnitudes(org):
    pass

def compute_magnitude_mb(org):

    # determine the magnitudes and store them in dictionaries.
    # XXX Requires a thorough clean-up!
    # Currently assumes ONE trigger per station

    mb     = {}
    amp_mb = {}
    per_mb = {}

    for arr in org.arr:
        if not arr.used:
            continue
#       if trg.pha != "P":
#           continue # XXX currently all triggers are considered P/PKP arrivals
        for trg in org.trgs:
            if arr.sta == trg.sta:
                break
        assert arr.sta == trg.sta
        dist = arr.dist
        try:
            m = bmagn(trg.ampl_mb, trg.peri_mb, dist)
            if m is None:
                continue
            mb[trg.sta] = m
            amp_mb[trg.sta] = trg.ampl_mb
            per_mb[trg.sta] = trg.peri_mb
        except UnboundLocalError:
            # XXX local variable 'dist' referenced before assignment
            continue
        except ValueError:
            continue

    return mb, amp_mb, per_mb
        

def compute_magnitude_ml(org):

    ml     = {}
    amp_ml = {}
    for arr in org.arr:
        if not arr.used:
            continue
        for trg in org.trgs:
            if arr.sta == trg.sta:
                break
        assert arr.sta == trg.sta
        dist = arr.dist
        try:
            ml[trg.sta] = lmagn(trg.ampl_ml, dist)
            amp_ml[trg.sta] = trg.ampl_ml
        except UnboundLocalError:
            # XXX local variable 'dist' referenced before assignment
            continue
        except ValueError:
            continue
        except AttributeError:
            continue

    return ml, amp_ml
