#!/usr/bin/env python

from Numeric        import sqrt, average
from seis.filters   import FFT_WoodAnderson as WoodAnderson
from seis.filters   import FFT_WWSSN_SP as WWSSN_SP
from seis.numeric   import measure_period, demean

from seiscomp.concat import concat

wwssn_sp = WWSSN_SP()
wood_and = WoodAnderson()

global ringbuf
ringbuf = {}

def measure_ampl(trc, t, len):
    from seis.numeric import find_max

    trc = trc.copy()
    trc.cut(0, len, t)
    imax, amax = find_max(abs(trc.data))

    return amax, imax

def measure_mb(detec, key):
    from seis.filters import demean
    trg = detec.pending[key]
##  buf = detec.buffer
##  trc = timeWindow(buf['BHZ'], trg.on-15, 60)

    cha = detec.cha[:-1] + "Z"

    ###
    id = "%s.%s.%s.%s.D" % (trg.net, trg.sta, trg.loc, cha)
    trc = timeWindow(ringbuf[id], trg.on-15, 40)
    ###

    if not trc: return
    demean(trc)
    wwssn_sp(trc)

    t0 = trg.on-1
    amax, imax = measure_ampl(trc, t0, 10)

#   try:
    pmax = measure_period(trc.data, int(trc.fsamp*(t0-trc.time))+imax, 1./trc.fsamp)
#   except IndexError:
#       pmax = -3.33
# XXX IndexError currently caught by __det_amp_mb

    gain = float(detec.config["gain"])
    amax = amax/gain
    tmax = t0 + imax/trc.fsamp
    return amax, tmax, pmax

def measure_ml(detec, key):
    from seis.filters import demean
    trg = detec.pending[key]
##  buf = detec.buffer

#   if 'BHN' not in buf or 'BHE' not in buf:
    if True:
        # This is Winfried's hack for computing ML from the Z
        # component only
##      trc = timeWindow(buf['BHZ'], trg.on-15, 180)
        cha = detec.cha[:-1] + "Z"
        ###
        id = "%s.%s.%s.%s.D" % (trg.net, trg.sta, trg.loc, cha)
        trc = timeWindow(ringbuf[id], trg.on-15, 150)
        ###

        if not trc: return
        demean(trc)
        wood_and(trc)

        t0 = trg.on-1
        amax, tmax = measure_ampl(trc, t0, 120)

        # XXX The gain is the same for all components. This may
        # XXX cause trouble for certain seismometers.
        gain = float(detec.config["gain"])
        amax = 2*amax/gain
        tmax = t0 + tmax/trc.fsamp

    return amax, tmax
    # XXX else measure it from sqrt(BHE**2+BHN**2)

def measure_polarization(detec, key):
    return None

    from seis.numeric   import demean
    from seis.polar     import polarization
    from seis.lib       import align # XXX

#   buf = detec.buffer

#   if 'BHZ' not in buf or 'BHN' not in buf or 'BHE' not in buf:
#       return # we need all 3 components to compute polarisation

    trg = detec.pending[key]

    traces = []
    for cha in ['Z', 'N', 'E']:
#       records = buf[cha].getTimeWindow(trg.on-15, 60)
        cha = detec.cha[:-1] + cha
        id = "%s.%s.%s.%s.D" % (trg.net, trg.sta, trg.loc, cha)
        if id not in ringbuf:
            # XXX issue a warning message here?
            return
        records = ringbuf[id].getTimeWindow(trg.on-15, 60)
        if not records:
            return

        trc = concat(records)
        filt = WWSSN_SP() # XXX we may want to use another filter
        trc.data = demean(trc.data)
        filt.apply(trc)
        trc.cut(-1, 4, trg.on)
        traces.append(trc)

    align(traces)

    fz, fn, fe = traces[0].data, traces[1].data, traces[2].data
    try:
        p = polarization(fe, fn, fz)
    except TypeError:
        # XXX appears to sometimes happen in polar.py, line 96, in polarization
        # XXX if eigvec[0][2]<0:
        # XXX TypeError: cannot compare complex numbers using <, <=, >, >=
        return
    except AssertionError: # XXX happens e.g. if time windows not aligned
        return
    except ZeroDivisionError: # XXX may happen in Numeric.average()
        return

    return p

def measure_exact_time(detec, key):
    # XXX XXX XXX XXX


    from seis.filters import demean, envelope
    trg = detec.pending[key]

    cha = detec.cha[:-1] + "Z"

    ###
    id = "%s.%s.%s.%s.D" % (trg.net, trg.sta, trg.loc, cha)
    trc = timeWindow(ringbuf[id], trg.on-1-20, 40)

    if not trc: return
    demean(trc)
    wwssn_sp(trc)
    envelope(trc)

    trc.cut(-20, 40, trg.on-1)
    env = trc.data
    noi = int((trg.on - 5 - trc.time)*trc.fsamp)
    rms = sqrt(average(env[:noi]**2))
    ion = int((trg.on - 0.5 - trc.time)*trc.fsamp)

    # search sample where env exceeds rms
    while env[ion]>rms:
        ion = ion - 1
    # search sample where env has next local minimum
    rms2 = rms/2
    while env[ion]>env[ion-1] and env[ion]>rms2:
        ion = ion - 1
    trg.on = trc.time + ion/trc.fsamp


def test_spike(detec, key):
    pass



def timeWindow(ringbuffer, start, length):

    records = ringbuffer.getTimeWindow(start, length)
    deco = [(r.time, r) for r in records]
    deco.sort()
    records = [r for t,r in deco]

    if not records:
        return None
    trc = concat(records)

    # make sure we got the *whole* time window requested
    if trc.time > start  or  trc.endtime < start+length:
        t = trc.time
        return None

    return trc
